@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -61,7 +61,10 @@ import com.openexchange.groupware.ldap.User;
 import com.openexchange.groupware.ldap.UserStorage;
 import com.openexchange.push.PushException;
 import com.openexchange.push.PushListener;
+import com.openexchange.push.PushUtility;
 import com.openexchange.push.mail.notify.services.PushServiceRegistry;
+import com.openexchange.session.Session;
+import com.openexchange.sessiond.SessionMatcher;
 import com.openexchange.sessiond.SessiondService;
 import com.openexchange.tools.iterator.ReadOnlyIterator;
 
@@ -71,12 +74,48 @@ import com.openexchange.tools.iterator.ReadOnlyIterator;
  */
 public final class MailNotifyPushListenerRegistry {
 
-    private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(MailNotifyPushListenerRegistry.class);
+    private static final org.apache.commons.logging.Log LOG = com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(MailNotifyPushListenerRegistry.class));
     
+    
+    /**
+     * @return the useOXLogin
+     */
+    public final boolean isUseOXLogin() {
+        return useOXLogin;
+    }
+
+    
+    /**
+     * @param useOXLogin the useOXLogin to set
+     */
+    public final void setUseOXLogin(boolean useOXLogin) {
+        this.useOXLogin = useOXLogin;
+    }
+
+    
+    /**
+     * @return the useEmailAddress
+     */
+    public final boolean isUseEmailAddress() {
+        return useEmailAddress;
+    }
+
+    
+    /**
+     * @param useEmailAddress the useEmailAddress to set
+     */
+    public final void setUseEmailAddress(boolean useEmailAddress) {
+        this.useEmailAddress = useEmailAddress;
+    }
+
     private static final MailNotifyPushListenerRegistry instance = new MailNotifyPushListenerRegistry();
 
     private final ConcurrentMap<String, MailNotifyPushListener> map;
 
+    private boolean useOXLogin;
+
+    private boolean useEmailAddress;
+
     /**
      * Initializes a new {@link MailNotifyPushListenerRegistry}.
      */
@@ -102,6 +141,7 @@ public final class MailNotifyPushListenerRegistry {
      */
     public void fireEvent(final String mboxid) throws PushException {
         final PushListener listener;
+        LOG.debug("checking whether to fire event for " + mboxid);
         if (null != (listener = map.get(mboxid))) {
             LOG.debug("fireEvent, mboxid=" + mboxid);
             listener.notifyNewMail();
@@ -117,15 +157,53 @@ public final class MailNotifyPushListenerRegistry {
      * @return <code>true</code> if push listener service could be successfully added; otherwise <code>false</code>
      */
     public boolean addPushListener(final int contextId, final int userId, final MailNotifyPushListener pushListener) throws PushException {
-        return (null == map.putIfAbsent(getMboxId(contextId, userId), pushListener));
+        boolean notYetPushed = true;
+        for(final String id : getMboxIds(contextId, userId)) {
+            LOG.debug("adding alias " + id + " to map");
+            final boolean pushFailed = (null == map.putIfAbsent(id, pushListener) ? false : true);
+            if( notYetPushed && pushFailed ) {
+                notYetPushed = false;
+            }
+        }
+        return notYetPushed;
     }
 
-    private String getMboxId(final int contextId, final int userId) throws PushException {
+    /**
+     * return all the users aliases with domain stripped off and localpart to lowercase
+     * 
+     * @param contextId
+     * @param userId
+     * @return String Array
+     * @throws PushException
+     */
+    private final String[] getMboxIds(final int contextId, final int userId) throws PushException {
         Context storageContext;
         try {
             storageContext = ContextStorage.getStorageContext(contextId);
             User user = UserStorage.getInstance().getUser(userId, storageContext);
-            return user.getLoginInfo();
+            int alength = user.getAliases().length;
+            if( useOXLogin ) {
+                alength++;
+            }
+            final String[] ret = new String[alength];
+            int i=0;
+            for(final String alias : user.getAliases()) {
+                if( useEmailAddress ) {
+                    ret[i] = alias.toLowerCase();
+                } else {
+                    final int idx = alias.indexOf("@");
+                    if( idx != -1) {
+                        ret[i] = alias.substring(0, idx).toLowerCase();
+                    } else {
+                        ret[i] = alias.toLowerCase();
+                    }
+                }
+                i++;
+            }
+            if( useOXLogin ) {
+                ret[i] = user.getLoginInfo().toLowerCase();
+            }
+            return ret;
         } catch (final ContextException e) {
             throw new PushException(e);
         } catch (final LdapException e) {
@@ -173,8 +251,8 @@ public final class MailNotifyPushListenerRegistry {
             try {
                 l.open();
             } catch (final PushException e) {
-                org.apache.commons.logging.LogFactory.getLog(MailNotifyPushListenerRegistry.class).error(
-                    MessageFormat.format("Opening MAL Poll listener failed. Removing listener from registry: {0}", l.toString()),
+                com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(MailNotifyPushListenerRegistry.class)).error(
+                    MessageFormat.format("Opening mail push UDP listener failed. Removing listener from registry: {0}", l.toString()),
                     e);
                 i.remove();
             }
@@ -190,7 +268,7 @@ public final class MailNotifyPushListenerRegistry {
      * @throws PushException 
      */
     public boolean purgeUserPushListener(final int contextId, final int userId) throws PushException {
-        return removeListener(getMboxId(contextId, userId));
+        return removeListener(getMboxIds(contextId, userId));
     }
 
     /**
@@ -204,18 +282,33 @@ public final class MailNotifyPushListenerRegistry {
      */
     public boolean removePushListener(final int contextId, final int userId) throws PushException {
         final SessiondService sessiondService = PushServiceRegistry.getServiceRegistry().getService(SessiondService.class);
-        if (null == sessiondService || 0 == sessiondService.getUserSessions(userId, contextId)) {
-            return removeListener(getMboxId(contextId, userId));
+        final String[] mboxIds = getMboxIds(contextId, userId);
+        // Bind MailNotifyPushListener to another session because of password change issues.
+        Session session = sessiondService.findFirstMatchingSessionForUser(userId, contextId, new SessionMatcher() {
+
+            public boolean accepts(Session tmp) {
+                return PushUtility.allowedClient(tmp.getClient());
+            }
+            
+        });
+        if (null != session) {
+            boolean retVal = removeListener(mboxIds);
+            final MailNotifyPushListener pushListener = MailNotifyPushListener.newInstance(session);
+            retVal &= addPushListener(contextId, userId, pushListener);
+            pushListener.open();
+            return retVal;
         }
-        return false;
+        return removeListener(mboxIds);
     }
 
-    private boolean removeListener(final String mboxId) {
-        final MailNotifyPushListener listener = map.remove(mboxId);
-        if (null != listener) {
-            listener.close();
-            return true;
+    private boolean removeListener(final String[] mboxIds) {
+        for(final String id : mboxIds) {
+            LOG.debug("removing alias" + id + " from map");
+            final MailNotifyPushListener listener = map.remove(id);
+            if (null != listener) {
+                listener.close();
+            }
         }
-        return false;
+        return true;
     }
 }
